searchData={"items":[{"type":"behaviour","title":"GenStatem","doc":"A behaviour module for implementing the generic state machine\nDocumentation for `GenStatem`.\n\n# Features\n\n- [Co-located state code](#state_functions)\n- [Arbitrary term state](#handle_event_function)\n- [Event postponing](#event-postponing)\n- [Self-generated events](#event-insertion)\n- [State time-out](`t:state_timeout/0`)\n- [Multiple generic named time-outs](`t:generic_timeout/0`)\n- [Absolute time-out time](`t:timeout_option/0`)\n- [Automatic state enter calls](#state-enter-calls)\n- [Reply from other state than the request](#reply-to-a-call), traceable with\n  [`:sys`](`:sys`)\n- [Multiple replies](#reply-to-a-call), traceable with [`:sys`](`:sys`)\n- [Changing the callback module](#change_callback_module)\n\nTwo [_callback modes_](`t:callback_mode/0`) are supported:\n\n- `state_functions` - for finite-state machine which requires the state\n  to be an atom and uses that state\n  as the name of the current callback function, arity 3.\n- `handle_event_function` - that allows the state to be any term\n  and that uses `c:handle_event/4` as callback function for all states.\n\n# Callback module\n\nA `GenStatem` assumes all specific parts to be located\nin a callback module exporting a predefined set of functions.\nThe relationship between the behavior functions\nand the callback functions is as follows:\n\n```\nGenStatem module            Callback module\n-----------------            ---------------\nGenStatem.start\nGenStatem.start_monitor\nGenStatem.start_link -----> Module.init/1\n\nServer start or code change\n                      -----> Module.callback_mode/0\n                      selects callback mode\n\nGenStatem.stop\nSupervisor exit\nCallback failure      -----> Module.terminate/3\n\nGenStatem.call\nGenStatem.cast\nGenStatem.send_request\nerlang.send\nerlang.'!'            -----> Module.state_name/3\n                   or -----> Module.handle_event/4\n                   depending on callback mode\n\nRelease upgrade/downgrade\n(code change)\n                      -----> Module.code_change/4\n```\n\n# State callback {: #state-callback }\n\nThe _state callback_ for a specific [state](`t:state/0`) in a `GenStatem`\nis the callback function that is called for all events in this state.\nIt is selected depending on which [_callback mode_](`t:callback_mode/0`)\nthat the callback module defines with the callback function\n[`Module.callback_mode/0`](`c:callback_mode/0`).\n\n[](){: #state_functions }\nWhen the [_callback mode_](`t:callback_mode/0`) is `state_functions`,\nthe state must be an atom and is used as the _state callback_ name;\nsee [`Module.state_name/3`](`c:state_name/3`).  This co-locates all code\nfor a specific state in one function as the `GenStatem` engine branches\ndepending on state name.  Note the fact that the callback function\n[`Module.terminate/3`](`c:terminate/3`) makes the state name `terminate`\nunusable in this mode.\n\n[](){: #handle_event_function }\nWhen the [_callback mode_](`t:callback_mode/0`) is `handle_event_function`,\nthe state can be any term and the _state callback_ name is\n[`Module.handle_event/4`](`c:handle_event/4`). This makes it easy\nto branch depending on state or event as you desire. Be careful about\nwhich events you handle in which states so that you do not accidentally\npostpone an event forever creating an infinite busy loop.\n\n# Event types\n\nEvents are of different [types](`t:event_type/0`),\ntherefore the callback functions can know the origin of an event\nwhen handling it.  [External events](`t:external_event_type/0`) are\n`:call`,  `:cast`, and `:info`. Internal events are\n[`timeout`](`t:timeout_event_type/0`) and `:internal`.\n\n# Event handling\n\nWhen `GenStatem` receives a process message it is transformed\ninto an event and the [_state callback_](#state-callback)\nis called with the event as two arguments: type and content. When the\n[_state callback_](#state-callback) has processed the event\nit returns to `GenStatem` which does a _state transition_. If this\n_state transition_ is to a different state, that is: `next_state !== state`,\nit is a _state change_.\n\n# Transition actions\n\nThe [_state callback_](#state-callback) may return\n[_transition actions_](`t:action/0`) for `GenStatem` to execute\nduring the _state transition_, for example to set a time-out\nor reply to a call.\n\n# Reply to a call {: #reply-to-a-call }\n\nSee [`GenStatem:call/2,3`](#call-reply) about how to reply\nto a call.  A reply can be sent from any _state callback_,\nnot just the one that got the request event.\n\n# Event postponing {: #event-postponing }\n\nOne of the possible _transition actions_ is to postpone the current event.\nThen it will not be handled in the current state.  The `GenStatem` engine\nkeeps a queue of events divided into postponed events and\nevents still to process (not presented yet).  After a _state change_\nthe queue restarts with the postponed events.\n\nThe `GenStatem` event queue model is sufficient to emulate\nthe normal process message queue with selective receive.\nPostponing an event corresponds to not matching it\nin a receive statement, and changing states corresponds to\nentering a new receive statement.\n\n# Event insertion {: #event-insertion }\n\nThe [_state callback_](#state-callback) can insert\nevents using the [_transition action_](`t:action/0`) `:next_event`,\nand such an event is inserted in the event queue as the next to call the\n[_state callback_](#state-callback) with. That is,\nas if it is the oldest incoming event. A dedicated `t:event_type/0`\n`:internal` can be used for such events making it possible to\nsafely distinguish them from external events.\n\nInserting an event replaces the trick of calling your own state handling\nfunctions that you often would have to resort to in, for example,\n`m:gen_fsm` to force processing an inserted event before others.\n\n> #### Note {: .info }\n>\n> If you postpone an event and (against good practice) directly call\n> a different _state callback_, the postponed event is not retried,\n> since there was no _state change_.\n>\n> Instead of directly calling a _state callback_, do a _state change_.\n> This makes the `GenStatem` engine retry postponed events.\n>\n> Inserting an event in a _state change_ also triggers\n> the new  _state callback_ to be called with that event\n> before receiving any external events.\n\n# State enter calls {: #state-enter-calls }\n\nThe `GenStatem` engine can automatically make a special call to the\n[_state callback_](#state-callback) whenever a new state is\nentered; see `t:state_enter/0`. This is for writing code common\nto all state entries.  Another way to do it is to explicitly insert\nan event at the _state transition_, and/or to use a dedicated\n_state transition_ function, but that is something you will have to\nremember at every _state transition_ to the state(s) that need it.\n\nFor the details of a _state transition_, see type `t:transition_option/0`.\n\n# Hibernation\n\nThe `GenStatem` process can go into hibernation;\nsee `proc_lib:hibernate/3`. It is done when\na [_state callback_](#state-callback) or\n[`Module.init/1`](`c:init/1`) specifies `hibernate`\nin the returned [`Actions`](`t:enter_action/0`) list. This feature\ncan be useful to reclaim process heap memory while the server\nis expected to be idle for a long time. However, use it with care,\nas hibernation can be too costly to use after every event;\nsee `:erlang.hibernate/3`.\n\nThere is also a server start option\n[`{hibernate_after, Timeout}`](`t:enter_loop_opt/0`)\nfor [`start/3,4`](`start/3`), [`start_link/3,4`](`start_link/3`),\n[`start_monitor/3,4`](`start_monitor/3`),\nthat may be used to automatically hibernate the server.\n\n# Callback failure\n\nIf a callback function fails or returns a bad value,\nthe `GenStatem` terminates.  However, an exception of class\n[`throw`](`:erlang.throw/1`) is not regarded as an error\nbut as a valid return, from all callback functions.\n\n# System messages and the `m:sys` module\n\nA `GenStatem` handles system messages as described in `m:sys`.\nThe `m:sys` module can be used for debugging a `GenStatem`.\nReplies sent through [_transition actions_](`t:action/0`)\ngets logged, but not replies sent through [`reply/1,2`](`reply/2`).\n\n# Trapping exit\n\nA `GenStatem` process, like all `gen_`* behaviours,\ndoes not trap exit signals automatically;\nthis must be explicitly initiated in the callback module\n(by calling [`process_flag(trap_exit, true)`](`:erlang.process_flag/2`)\npreferably from `c:init/1`.\n\n# Server termination\n\nIf the `GenStatem` process terminates, e.g. as a result\nof a callback function returning `{stop, Reason}`, an exit signal\nwith this `Reason` is sent to linked processes and ports.\nSee [Processes](https://www.erlang.org/doc/system/ref_man_processes.html#error-handling)\nin the Reference Manual for details regarding error handling\nusing exit signals.\n\n> #### Note {: .info }\n>\n> For some important information about distributed signals, see the\n> [_Blocking Signaling Over Distribution_\n> ](https://www.erlang.org/doc/system/ref_man_processes#blocking-signaling-over-distribution)\n> section in the _Processes_ chapter of the _Erlang Reference Manual_.\n> Blocking signaling can, for example, cause call time-outs in `GenStatem`\n> to be significantly delayed.\n\n# Bad argument\n\nUnless otherwise stated, all functions in this module fail if the specified\n`GenStatem` does not exist or if bad arguments are specified.","ref":"GenStatem.html"},{"type":"behaviour","title":"Example - GenStatem","doc":"The following example shows a simple pushbutton model\nfor a toggling pushbutton implemented with\n[_callback mode_](`t:callback_mode/0`) `state_functions`.\nYou can push the button and it replies if it went on or off,\nand you can ask for a count of how many times it has been pushed\nto switch on.","ref":"GenStatem.html#module-example"},{"type":"behaviour","title":"Pushbutton State Diagram - GenStatem","doc":"```mermaid\n---\ntitle: Pushbutton State Diagram\n---\nstateDiagram-v2\n    [*]  --> off\n    off  --> on  : push   Increment count  Reply 'on'  \n    on   --> off : push   Reply 'off'  \n```\n\nNot shown in the state diagram:\n* The API function `push()` generates an event `push` of type `call`.\n* The API function `get_count()` generates an event `get_count`\n  of type `call` that is handled in all states by replying with\n  the current count value.\n* Unknown events are ignored and discarded.\n* There is boilerplate code for start, stop, terminate, code change,\n  init, to set the callback mode to `state_functions`, etc...","ref":"GenStatem.html#module-pushbutton-state-diagram"},{"type":"behaviour","title":"Pushbutton Code - GenStatem","doc":"The following is the complete callback module file `pushbutton.erl`:\n\n```elixir\ndefmodule PushButton do\n  use GenStatem\n\n  defp name, do: :pushbutton_statem","ref":"GenStatem.html#module-pushbutton-code"},{"type":"behaviour","title":"API. This example uses a registered name name() - GenStatem","doc":"","ref":"GenStatem.html#module-api-this-example-uses-a-registered-name-name"},{"type":"behaviour","title":"and does not link to the caller - GenStatem","doc":"def start, do: GenStatem.start(__MODULE__, [], name: name())\n\n  def push, do: GenStatem.call(name(), :push)\n\n  def get_count, do: GenStatem.call(name(), :get_count)\n\n  def stop, do: GenStatem.stop(name())","ref":"GenStatem.html#module-and-does-not-link-to-the-caller"},{"type":"behaviour","title":"Mandatory callback functions - GenStatem","doc":"def init([]) do","ref":"GenStatem.html#module-mandatory-callback-functions"},{"type":"behaviour","title":"Set the initial state + data.  Data is used only as a counter. - GenStatem","doc":"state = :off\n    data = 0\n    {:ok, state, data}\n  end\n\n  def callback_mode(), do: :state_functions\n\n  def code_change(_old_vsn, old_state, old_data, _extra) do\n\t{:ok, old_state, old_data}\n  end","ref":"GenStatem.html#module-set-the-initial-state-data-data-is-used-only-as-a-counter"},{"type":"behaviour","title":"Mandatory callback functions - GenStatem","doc":"def terminate, do: :void","ref":"GenStatem.html#module-mandatory-callback-functions-1"},{"type":"behaviour","title":"state callback(s) - GenStatem","doc":"def off({:call, from}, :push, data) do","ref":"GenStatem.html#module-state-callback-s"},{"type":"behaviour","title":"Go to 'on', increment count and reply - GenStatem","doc":"","ref":"GenStatem.html#module-go-to-on-increment-count-and-reply"},{"type":"behaviour","title":"that the resulting status is 'on' - GenStatem","doc":"{:next_state, :on, data + 1,[{:reply, from, :on}]}\n  end\n\n  def off(event_type, event_content, data) do\n    handle_event(event_type, event_content, data)\n  end\n\n  def on({:call, from}, :push, data) do","ref":"GenStatem.html#module-that-the-resulting-status-is-on"},{"type":"behaviour","title":"Go to 'off' and reply that the resulting status is 'off' - GenStatem","doc":"{:next_state, :off, data, [{:reply, from, :off}]}\n  end\n\n  def on(event_type, event_content, data) do\n    handle_event(event_type, event_content, data)\n  end","ref":"GenStatem.html#module-go-to-off-and-reply-that-the-resulting-status-is-off"},{"type":"behaviour","title":"Handle events common to all states - GenStatem","doc":"defp handle_event({:call, from}, :get_count, data) do","ref":"GenStatem.html#module-handle-events-common-to-all-states"},{"type":"behaviour","title":"Reply with the current count - GenStatem","doc":"{:keep_state, data,[{:reply, from, data}]}\n  end\n  defp handle_event(_event_type, _event_content, data) do","ref":"GenStatem.html#module-reply-with-the-current-count"},{"type":"behaviour","title":"Ignore all other events - GenStatem","doc":"{:keep_state, data}\n  end\nend\n```\n\nThe following is a shell session when running it:\n\n```elixir\niex(1)> PushButton.start\n{:ok, #PID<0.203.0>}\niex(2)> PushButton.get_count\n0\niex(3)> PushButton.push\n:on\niex(4)> PushButton.get_count\n1\niex(5)> PushButton.push\n:off\niex(6)> PushButton.get_count\n1\niex(7)> PushButton.stop\n:ok\niex(8)> PushButton.push\n** (GenStatem.GenError)\nClass: exit\nReason: noproc\nMFArgs: GenStatem.call(:pushbutton_statem, :push, :infinity)\nStacktrace:\n    (stdlib 6.2.2) gen.erl:580: :gen.do_for_proc/2\n    (GenStatem 0.1.0) lib/GenStatem.ex:1997: GenStatem.call/3\n    (elixir 1.18.3) src/elixir.erl:386: :elixir.eval_external_handler/3\n    (stdlib 6.2.2) erl_eval.erl:919: :erl_eval.do_apply/7\n    (elixir 1.18.3) src/elixir.erl:364: :elixir.eval_forms/4\n    (elixir 1.18.3) lib/module/parallel_checker.ex:120: Module.ParallelChecker.verify/1\n    (iex 1.18.3) lib/iex/evaluator.ex:336: IEx.Evaluator.eval_and_inspect/3\n    (iex 1.18.3) lib/iex/evaluator.ex:310: IEx.Evaluator.eval_and_inspect_parsed/3\n\n    (GenStatem 0.1.0) lib/GenStatem.ex:2001: GenStatem.call/3\n    iex:8: (file)\n```\n\nTo compare styles, here follows the same example using\n[_callback mode_](`t:callback_mode/0`) `handle_event_function`,\nor rather, the code to replace after function [`init/1`](`c:init/1`)\nof the `pushbutton.erl` example file above:\n\n```elixir","ref":"GenStatem.html#module-ignore-all-other-events"},{"type":"behaviour","title":"state callback(s) - GenStatem","doc":"def handle_event({:call, from}, :push, :off, data) do","ref":"GenStatem.html#module-state-callback-s-1"},{"type":"behaviour","title":"Go to 'on', increment count and reply - GenStatem","doc":"","ref":"GenStatem.html#module-go-to-on-increment-count-and-reply-1"},{"type":"behaviour","title":"that the resulting status is 'on' - GenStatem","doc":"{:next_state, :on, data + 1,[{:reply, from, :on}]}\nend\n\ndef handle_event({:call, from}, :push, :on, data) do","ref":"GenStatem.html#module-that-the-resulting-status-is-on-1"},{"type":"behaviour","title":"Go to 'off' and reply that the resulting status is 'off' - GenStatem","doc":"{:next_state, :off, data, [{:reply, from, :off}]}\nend\n\n##","ref":"GenStatem.html#module-go-to-off-and-reply-that-the-resulting-status-is-off-1"},{"type":"behaviour","title":"Event handling common to all states - GenStatem","doc":"def handle_event({:call, from}, :get_count, state, data) do","ref":"GenStatem.html#module-event-handling-common-to-all-states"},{"type":"behaviour","title":"Reply with the current count - GenStatem","doc":"{:next_state, state, data, [{:reply, from, data}]};\nend\n\ndef handle_event(_event_type, _old_state, current_state, data) do","ref":"GenStatem.html#module-reply-with-the-current-count-1"},{"type":"behaviour","title":"Ignore all other events - GenStatem","doc":"{:next_state, current_state, data}\nend\n```","ref":"GenStatem.html#module-ignore-all-other-events-1"},{"type":"behaviour","title":"See Also - GenStatem","doc":"`m:GenEvent`, `m:GenServer`, `m:Supervisor`.","ref":"GenStatem.html#module-see-also"},{"type":"type","title":"GenStatem.action/0","doc":"Actions for a _state transition_, or when starting the server.\n\nThese _transition actions_ can be invoked by returning them from the\n[_state callback_](#state-callback) when it is called\nwith an [event](`t:event_type/0`), from [`Module.init/1`](`c:init/1`).\nThey are **not allowed** from _state enter calls_.\n\nActions are executed in the containing list order.\n\nActions that set [transition options](`t:transition_option/0`)\noverride any previous of the same type, so the last\nin the containing list wins.  For example, the last `t:postpone/0`\noverrides any previous `t:postpone/0` in the list.\n\n- **`{:postpone, value}`** - Sets the\n  [`transition_option()` ](`t:transition_option/0`)`t:postpone/0`\n  for this _state transition_.  This action is ignored when returned from\n  [`Module.init/1`](`c:init/1`), as there is no event to postpone\n  in those cases.\n\n  `:postpone` is equivalent to `{:postpone, true}`.\n\n- **`{:next_event, event_type, event_content}`** - This action\n  does not set any [`transition_option()`](`t:transition_option/0`)\n  but instead stores the specified `event_type` and `event_content`\n  for insertion after all actions have been executed.\n\n  The stored events are inserted in the queue as the next to process\n  before any already queued events. The order of these stored events\n  is preserved, so the first `:next_event` in the containing list\n  becomes the first to process.\n\n  An event of type [`internal`](`t:event_type/0`) should be used\n  when you want to reliably distinguish an event inserted this way\n  from any external event.\n\n- **`{:change_callback_module, new_module}`** {: #change_callback_module } -\n  Changes the callback module to `new_module` which will be used\n  when calling all subsequent [state callbacks](#state-callback).\n  The `GenStatem` engine will find out the\n  [_callback mode_](`t:callback_mode/0`) of `new_module` by calling\n  [`new_module.callback_mode/0`](`c:callback_mode/0`) before the next\n  [state callback](#state-callback).\n\n  Changing the callback module does not affect the _state transition_\n  in any way, it only changes which module that handles the events.\n  Be aware that all relevant callback functions in `new_module` such as\n  the [state callback](#state-callback),\n  [`new_module.code_change/4`](`c:code_change/4`),\n  [`new_module.format_status/1`](`c:format_status/1`) and\n  [`new_module.terminate/3`](`c:terminate/3`) must be able to handle\n  the state and data from the old module.\n\n- **`{:push_callback_module, new_module}`** {: #push_callback_module } -\n   Pushes the current callback module to the top of an internal stack\n   of callback modules, and changes the callback module to `new_module`.\n   Otherwise like `{:change_callback_module, new_module}` above.\n\n- **`pop_callback_module`** {: #pop_callback_module } -\n  Pops the top module from the internal stack of callback modules\n  and changes the callback module to be the popped module.\n  If the stack is empty the server fails.\n  Otherwise like `{:change_callback_module, new_module}` above.","ref":"GenStatem.html#t:action/0"},{"type":"function","title":"GenStatem.call/3","doc":"Call a server: send request and wait for response.\n\nMakes a synchronous call to the `GenStatem`\n[`ServerRef`](`t:server_ref/0`) by sending a request\nand waiting until the response arrives.\n\n[](){: #call-reply }\nThe `GenStatem` calls the\n[_state callback_](#state-callback)\nwith `t:event_type/0` `{:call, from}` and event content `request`.\n\nThe server's reply is sent from a [_state callback_](#state-callback),\nby returning a [_transition action_](`t:action/0`) `{:reply, from, reply}`,\ncalling [`reply(replies)`](`reply/1`) with such a reply action\nin the `replies` list, or calling [`reply(from, reply)`](`reply/2`).\n\n`timeout` is an integer > 0, which specifies how many milliseconds\nto wait for a reply, or the atom `:infinity` to wait indefinitely,\nwhich is the default. If no reply is received within the specified time,\nthe function call fails.\n\nThe call can also fail, for example, if the `GenStatem`\ndies before or during this function call.\n\nWhen this call fails it raises the [`GenError`](`m:GenStatem.GenError`) Exception and exit\nthe calling process.","ref":"GenStatem.html#call/3"},{"type":"callback","title":"GenStatem.callback_mode/0","doc":"Select the _callback mode_ and possibly\n[_state enter calls_](`t:state_enter/0`).\n\nThis function is called by a `GenStatem` when it needs to find out the\n[_callback mode_](`t:callback_mode/0`) of the callback module.\n\nThe value is cached by `GenStatem` for efficiency reasons,\nso this function is only called once after server start,\nafter code change, and after changing the callback module,\nbut before the first [_state callback_](#state-callback)\nin the current callback module's code is called.  More occasions may be\nadded in future versions of `GenStatem`.\n\nServer start happens either when [`Module.init/1`](`c:init/1`) returns.\nCode change happens when [`Module.code_change/4`](`c:code_change/4`)\nreturns.  A change of the callback module happens when\na [_state callback_](#state-callback) returns\nany of the actions [`change_callback_module`](#push_callback_module),\n[`push_callback_module`](#push_callback_module) or\n[`pop_callback_module`](#pop_callback_module).\n\nThe `callback_mode` is either just `t:callback_mode/0`\nor a list containing `t:callback_mode/0` and possibly\nthe atom [`:state_enter`](`t:state_enter/0`).\n\n> #### Note {: .info }\n>\n> If this function's body does not return an inline constant value\n> the callback module is doing something strange.","ref":"GenStatem.html#c:callback_mode/0"},{"type":"type","title":"GenStatem.callback_mode/0","doc":"One function per state or one common event handler.\n\nThe _callback mode_ is selected with the return value from\n[`Module.callback_mode/0`](`c:callback_mode/0`):\n\n- **`state_functions`** - The state must be of type `t:state_name/0`\n  and one callback function per state, that is,\n  [`Module.state_name/3`](`c:state_name/3`), is used.\n\n- **`handle_event_function`** - The state can be any term and the callback\n  function [`Module.handle_event/4`](`c:handle_event/4`)\n  is used for all states.\n\nThe function [`Module.callback_mode/0`](`c:callback_mode/0`) is called\nwhen starting the `GenStatem`, after code change and after changing\nthe callback module with any of the actions\n[`change_callback_module`](#change_callback_module),\n[`push_callback_module`](#push_callback_module),\nor [`pop_callback_module`](#pop_callback_module).\nThe result is cached for subsequent calls to\n[_state callbacks_](#state-callback).","ref":"GenStatem.html#t:callback_mode/0"},{"type":"type","title":"GenStatem.callback_mode_result/0","doc":"Return value from [`Module.callback_mode/0`](`c:callback_mode/0`).\n\nThis is the return type from\n[`Module.callback_mode/0`](`c:callback_mode/0`)\nwhich selects [_callback mode_](`t:callback_mode/0`)\nand whether to do [_state enter calls_](`t:state_enter/0`),\nor not.","ref":"GenStatem.html#t:callback_mode_result/0"},{"type":"function","title":"GenStatem.cast/2","doc":"Cast an event to a server.\n\nSends an asynchronous `cast` event to the `GenStatem`\n[`ServerRef`](`t:server_ref/0`) and returns `ok` immediately,\nignoring if the destination node or `GenStatem` does not exist.\n\nThe `GenStatem` calls the\n[_state callback_](#state-callback)\nwith `t:event_type/0` `cast` and event content `Msg`.","ref":"GenStatem.html#cast/2"},{"type":"function","title":"GenStatem.check_response/2","doc":"Check if a received message is a request response.\n\nChecks if `msg` is a response corresponding to the request identifier `req_id`.\nThe request must have been made by `send_request/2` and by the same process\ncalling this function.\n\nIf `msg` is a reply to the handle `req_id` the result of the request is returned\nin `reply`.  Otherwise this function returns `:no_reply` and no cleanup is done,\nand thus the function shall be invoked repeatedly until the response is returned.\n\nSee [`call/3`](#call-reply) about how the request is handled and the `reply` is\nsent by the `GenStatem` server.\n\nIf the `GenStatem` server process has died when this function is called, that\nis; `msg` reports the server's death, this function returns an `error` return\nwith the exit `reason`.","ref":"GenStatem.html#check_response/2"},{"type":"function","title":"GenStatem.check_response/3","doc":"","ref":"GenStatem.html#check_response/3"},{"type":"callback","title":"GenStatem.code_change/4","doc":"Update the [state](`t:state/0`) and [data](`t:data/0`) after code change.\n\nThis function is called by a `GenStatem` when it is to update\nits internal state during a release upgrade/downgrade that is,\nwhen the instruction `{:update, module, change, ...}`,\nwhere `change = {:advanced, extra}`, is specified in\nthe [`appup`](https://www.erlang.org/doc/apps/sasl/appup.html) file.\nFor more information, see\n[Release Handling Instructions in OTP Design Principles\n](https://www.erlang.org/docs/28/system/release_handling.html#release-handling-principles).\n\nFor an upgrade, `old_vsn` is `vsn`, and for a downgrade, `old_vsn` is\n`{:down, vsn}`. `vsn` is defined by the `vsn` attribute(s)\nof the old version of the callback module `Module`.\nIf no such attribute is defined, the version is the checksum\nof the Beam file.\n\n`old_state` and `old_data` are the internal state of the `GenStatem`.\n\n`extra` is passed \"as is\" from the `{:advanced, extra}` part\nof the update instruction.\n\nIf successful, the function must return the updated internal state\nin an `{ok, new_state, new_data}` tuple.\n\nIf the function returns a failure `reason`, the ongoing upgrade fails\nand rolls back to the old release. Note that `reason` cannot be\nan `{:ok, _, _}` tuple since that will be regarded\nas a `{:ok, new_state, new_data}` tuple, and that a tuple matching `{:ok, _}`\nis an also invalid failure `reason`. It is recommended to use\nan atom as `reason` since it will be wrapped in an `{:error, reason}` tuple.\n\nAlso note when upgrading a `GenStatem`, this function and hence the\n`change = {:advanced, extra}` parameter\nin the [`appup`](https://www.erlang.org/doc/apps/sasl/appup.html) file is not only\nneeded to update the internal state or to act on the `extra`\nargument. It is also needed if an upgrade or downgrade should change\n[_callback mode_](`t:callback_mode/0`), or else the _callback mode_\nafter the code change will not be honoured, most probably causing\na server crash.\n\nIf the server changes callback module using any of the actions\n[`change_callback_module`](#change_callback_module),\n[`push_callback_module`](#push_callback_module), or\n[`pop_callback_module`](#pop_callback_module), be aware that it is always\nthe current callback module that will get this callback call.\nThat the current callback module handles the current\nstate and data update should be no surprise, but it\nmust be able to handle even parts of the state and data\nthat it is not familiar with, somehow.\n\nIn the supervisor [child specification\n](https://www.erlang.org/docs/28/system/sup_princ.html#child-specification)\nthere is a list of modules which is recommended to contain\nonly the callback module. For a `GenStatem` with multiple callback modules\nthere is no real need to list all of them, it may not even be possible since\nthe list could change after code upgrade.  If this list would contain only\nthe start callback module, as recommended, what is important\nis to upgrade _that_ module whenever a _synchronized code replacement_ is done.\nThen the release handler concludes that an upgrade that upgrades _that_ module\nneeds to suspend, code change, and resume any server whose child specification\ndeclares that it is using _that_ module.\nAnd again; the _current_ callback module will get the\n[`Module.code_change/4`](`c:code_change/4`) call.\n\n> #### Note {: .info }\n>\n> If a release upgrade/downgrade with `change = {:advanced, extra}`\n> specified in the `.appup` file is made\n> when [`Module.code_change/4`](`c:code_change/4`) is not implemented\n> the process will crash with exit reason `undef`.","ref":"GenStatem.html#c:code_change/4"},{"type":"type","title":"GenStatem.data/0","doc":"Generic state data for the server.\n\nA term in which the state machine implementation is to store\nany server data it needs. The difference between this and the `t:state/0`\nitself is that a change in this data does not cause postponed events\nto be retried. Hence, if a change in this data would change\nthe set of events that are handled, then that data item\nshould be part of the `t:state/0` instead.","ref":"GenStatem.html#t:data/0"},{"type":"type","title":"GenStatem.enter_action/0","doc":"Actions for any callback: hibernate, time-outs or replies.\n\nThese _transition actions_ are allowed when a `t:action/0` is allowed,\nand also from a _state enter call_, and can be invoked\nby returning them from the [_state callback_](#state-callback), from\n[`Module.init/1`](`c:init/1`).\n\nActions are executed in the containing list order.\n\nActions that set [transition options](`t:transition_option/0`)\noverride any previous of the same type, so the last in the containing\nlist wins. For example, the last `t:event_timeout/0` overrides any previous\n`t:event_timeout/0` in the list.\n\n- **`{:hibernate, value}`** - Sets the `t:transition_option/0`\n  `t:hibernate/0` for this _state transition_.\n\n  `hibernate` is equivalent to `{:hibernate, true}`.","ref":"GenStatem.html#t:enter_action/0"},{"type":"type","title":"GenStatem.enter_loop_opt/0","doc":"Server [start options](#start-options) for the\n[`start/3,4`](`start/3`), [`start_link/3,4`](`start_link/3`),\nand [`start_monitor/3,4`](`start_monitor/3`), functions.\n\nSee [`start_link/4`](#start-options).","ref":"GenStatem.html#t:enter_loop_opt/0"},{"type":"type","title":"GenStatem.event_content/0","doc":"Event payload from the event's origin, delivered to\nthe [_state callback_](#state-callback).\n\nSee [`event_type`](`t:event_type/0`) that describes the origins of\nthe different event types, which is also where the event's content\ncomes from.","ref":"GenStatem.html#t:event_content/0"},{"type":"type","title":"GenStatem.event_handler_result/1","doc":"","ref":"GenStatem.html#t:event_handler_result/1"},{"type":"type","title":"GenStatem.event_handler_result/2","doc":"Return value from a [_state callback_](#state-callback)\nafter handling an event.\n\n`state_type` is `t:state_name/0`\nif [_callback mode_](`t:callback_mode/0`) is `:state_functions`,\nor `t:state/0`\nif [_callback mode_](`t:callback_mode/0`) is `:handle_event_function`.\n\n- **`{:next_state, next_state, new_data [, actions]}`** -\n  The `GenStatem` does a _state transition_ to `next_state`\n  (which may be the same as the current state), sets `new_data`\n  as the current server `t:data/0`, and executes all `actions`.\n  If `NextState !== CurrentState` the _state transition_\n  is a _state change_.","ref":"GenStatem.html#t:event_handler_result/2"},{"type":"type","title":"GenStatem.event_timeout/0","doc":"How long to wait for an event.\n\nStarts a timer set by `t:timeout_action/0`\n`Time`, or `{timeout, Time, EventContent [, Options]}`.\n\nWhen the timer expires an event of `t:event_type/0` `timeout`\nwill be generated. See `:erlang.start_timer/4` for how `Time`\nand [`Options`](`t:timeout_option/0`) are interpreted.  Future\n`:erlang.start_timer/4` `Options` will not necessarily be supported.\n\nAny event that arrives cancels this time-out. Note that a retried\nor inserted event counts as arrived. So does a state time-out zero event,\nif it was generated before this time-out is requested.\n\nIf `Time` is `infinity`, no timer is started,\nas it never would expire anyway.\n\nIf `Time` is relative and `0` no timer is actually started,\ninstead the the time-out event is enqueued to ensure\nthat it gets processed before any not yet received external event,\nbut after already queued events.\n\nNote that it is not possible nor needed to cancel this time-out,\nas it is cancelled automatically by any other event, meaning that\nwhenever a callback is invoked that may want to cancel this time-out,\nthe timer is already cancelled or expired.\n\nThe timer `EventContent` can be updated with the\n[`{timeout, update, NewEventContent}`](`t:timeout_update_action/0`)\naction without affecting the time of expiry.","ref":"GenStatem.html#t:event_timeout/0"},{"type":"type","title":"GenStatem.event_type/0","doc":"All event types: [external](`t:external_event_type/0`),\n[time-out](`t:timeout_event_type/0`), or `internal`.\n\n`internal` events can only be generated by the state machine itself\nthrough the _transition action_ [`next_event`](`t:action/0`).","ref":"GenStatem.html#t:event_type/0"},{"type":"type","title":"GenStatem.external_event_type/0","doc":"Event from a [call](`call/3`), [cast](`cast/2`),\nor regular process message; \"info\".\n\nType `{:call, from}` originates from the API functions\n[`call/2,3`](`call/3`) or `send_request/2`. The event contains\n[`From`](`t:from/0`), which is whom to reply to\nby a `t:reply_action/0` or [`reply/2,3`](`reply/2`) call.\n\nType `cast` originates from the API function `cast/2`.\n\nType `info` originates from regular process messages\nsent to the `GenStatem` process.","ref":"GenStatem.html#t:external_event_type/0"},{"type":"type","title":"GenStatem.format_status/0","doc":"A map that describes the server's status.\n\nThe keys are:\n- **`state`** - The current state.\n- **`data`** - The state data.\n- **`reason`** - The reason that caused the process to terminate.\n- **`queue`** - The event queue.\n- **`postponed`** - The queue of [postponed](`t:postpone/0`) events.\n- **`timeouts`** - The active [time-outs](`t:timeout_action/0`).\n- **`log`** - The [sys log](`:sys.log/2`) of the server.\n\nNew associations may be added to the status map without prior notice.","ref":"GenStatem.html#t:format_status/0"},{"type":"callback","title":"GenStatem.format_status/1","doc":"Format/limit the status value.\n\nThis function is called by a `GenStatem` process\nin order to format/limit the server status\nfor debugging and logging purposes.\n\nIt is called in the following situations:\n\n- [`:sys.get_status/1,2`](`:sys.get_status/1`) is invoked\nto get the `GenStatem` status.\n- The `GenStatem` process terminates abnormally and logs an error.\n\nThis function is useful for changing the form and appearance\nof the `GenStatem` status for these cases.  A callback module\nwishing to change the [`:sys.get_status/1,2`](`:sys.get_status/1`)\nreturn value and how its status appears in termination error logs,\nexports an instance of [`Module.format_status/1`](`c:format_status/1`),\nwhich will get a map `Status` that describes the current state\nof the `GenStatem`, and shall return a map `NewStatus`\ncontaining the same keys as the input map,\nbut it may transform some values.\n\nOne use case for this function is to return compact alternative state\nrepresentations to avoid having large state terms printed in log files.\nAnother is to hide sensitive data from being written to the error log.\n\nExample:\n\n```elixir\ndef format_status(status) when is_map(status) do\n  status\n  |> Enum.map(fn\n    {:state, state} when is_map(state) ->\n      {:state, Map.delete(state, :private_key)}\n    {:message, {:password, _pass}} ->\n      {:message, {:password, :removed}}\n    pair ->\n      pair\n  end)\n  |> Map.new()\nend\n```\n\n> #### Note {: .info }\n>\n> This callback is optional, so a callback module does not need\n> to export it.  The `GenStatem` module provides\n> a default implementation of this function that returns `{state, data}`.","ref":"GenStatem.html#c:format_status/1"},{"type":"type","title":"GenStatem.from/0","doc":"A [`call`](`t:external_event_type/0`) event's reply destination.\n\nDestination to use when replying through, for example,\nthe action [`{:reply, from, reply}`](`t:reply_action/0`)\nto a process that has called the `GenStatem` server\nusing [`call/2,3`](`call/3`).","ref":"GenStatem.html#t:from/0"},{"type":"type","title":"GenStatem.generic_timeout/0","doc":"How long to wait for a named time-out event.\n\nStarts a timer set by `t:timeout_action/0`\n`{{timeout, Name}, Time, EventContent [, Options]}`.\n\nWhen the timer expires an event of `t:event_type/0` `{timeout, Name}`\nwill be generated. See `:erlang.start_timer/4` for how `Time`\nand [`Options`](`t:timeout_option/0`) are interpreted. Future\n`:erlang.start_timer/4` `Options` will not necessarily be supported.\n\nIf `Time` is `infinity`, no timer is started,\nas it never would expire anyway.\n\nIf `Time` is relative and `0` no timer is actually started,\ninstead the time-out event is enqueued to ensure\nthat it gets processed before any not yet received external event.\n\nSetting a timer with the same `Name` while it is running\nwill restart it with the new time-out value.  Therefore it is possible\nto cancel a specific time-out by setting it to `infinity`.\nIt can also be cancelled more explicitly with the\n[`{{timeout, Name}, cancel}`](`t:timeout_cancel_action/0`) action.\n\nThe timer `EventContent` can be updated with the\n[`{{timeout, Name}, update, NewEventContent}`](`t:timeout_update_action/0`)\naction without affecting the time of expiry.","ref":"GenStatem.html#t:generic_timeout/0"},{"type":"callback","title":"GenStatem.handle_event/4","doc":"[_state callback_](#state-callback) in\n[_callback mode_](`t:callback_mode/0`) `handle_event_function`.\n\nWhenever a `GenStatem` receives an event from [`call/2,3`](`call/3`),\n`cast/2`, or as a normal process message, this function is called.\n\nIf `event_type` is [`{:call, from}`](`t:event_type/0`),\nthe caller waits for a reply.  The reply can be sent from this\nor from any other [_state callback_](#state-callback)\nby returning with `{:reply, from, reply}` in [`actions`](`t:action/0`),\nin [`replies`](`t:reply_action/0`), or by calling\n[`reply(from, reply)`](`reply/2`).\n\nIf this function returns with a next state\nthat does not match equal (`!==`) to the current state,\nall postponed events are retried in the next state.\n\nFor options that can be set and actions that can be done\nby `GenStatem` after returning from this function, see `t:action/0`.\n\nWhen the `GenStatem` runs with [_state enter calls_](`t:state_enter/0`),\nthis function is also called with arguments `(:enter, old_state, ...)`\nduring every _state change_.  In this case there are some restrictions\non the [actions](`t:action/0`) that may be returned:\n\n- `t:postpone/0` is not allowed since a _state enter call_\nis not an event so there is no event to postpone.\n- [`{:next_event, _, _}`](`t:action/0`) is not allowed since\nusing _state enter calls_ should not affect how events\nare consumed and produced.\n- It is not allowed to change states from this call.\nShould you return `{:next_state, next_state, ...}`\nwith `next_state !== state` the `GenStatem` crashes.\n\nNote that it is actually allowed to use `{:repeat_state, new_data, ...}`\nalthough it makes little sense since you immediately\nwill be called again with a new _state enter call_ making this\njust a weird way of looping, and there are better ways to loop.\n\nIf you do not update `new_data` and have some loop termination condition,\nor if you use `{:repeat_state_and_data, _}` or `:repeat_state_and_data`\nyou have an infinite loop!\n\nYou are advised to use `{:keep_state, ...}`, `{:keep_state_and_data, _}`\nor `:keep_state_and_data` since changing states\nfrom a _state enter call_ is not possible anyway.\n\nNote the fact that you can use [`throw`](`:erlang.throw/1`)\nto return the result, which can be useful.  For example to bail out with\n[`throw(:keep_state_and_data)`](`throw/1`) from deep within complex code\nthat cannot return `{:next_state, state, data}` because `state` or `data`\nis no longer in scope.","ref":"GenStatem.html#c:handle_event/4"},{"type":"type","title":"GenStatem.hibernate/0","doc":"Hibernate the server process.\n\nIf `true`, hibernates the `GenStatem` by calling `proc_lib:hibernate/3`\nbefore going into `receive` to wait for a new external event.\n\nThere is also a server start option\n[`{hibernate_after, Timeout}`](`t:enter_loop_opt/0`)\nfor automatic hibernation.\n\n> #### Note {: .info }\n>\n> If there are enqueued events to process when hibernation is requested,\n> this is optimized by not hibernating but instead calling\n> [`:erlang.garbage_collect/0`](`:erlang.garbage_collect/0`) to simulate,\n> in a more efficient way, that the `GenStatem` entered hibernation\n> and immediately got awakened by an enqueued event.","ref":"GenStatem.html#t:hibernate/0"},{"type":"callback","title":"GenStatem.init/1","doc":"Initialize the state machine.\n\nWhenever a `GenStatem` is started using\n[`start_link/3,4`](`start_link/3`),\n[`start_monitor/3,4`](`start_monitor/3`), or\n[`start/3,4`](`start/3`), this function is called by the new process\nto initialize the implementation state and server data.\n\n`Args` is the `Args` argument provided to that start function.\n\n> #### Note {: .info }\n>\n> Note that if the `GenStatem` is started through `m:proc_lib`, this\n> callback will never be called.  Since this callback is not optional\n> it can in that case be implemented as:\n>\n> ```erlang\n> -spec init(_) -> no_return().\n> init(Args) -> :erlang.error(not_implemented, [Args]).\n> ```","ref":"GenStatem.html#c:init/1"},{"type":"type","title":"GenStatem.init_result/1","doc":"","ref":"GenStatem.html#t:init_result/1"},{"type":"type","title":"GenStatem.init_result/2","doc":"The return value from [`Module.init/1`](`c:init/1`).\n\nFor a successful initialization, `state` is the initial `t:state/0`,\nand `data` the initial server `t:data/0` of the `GenStatem`.\n\nThe [`actions`](`t:action/0`) are executed when entering the first\n[state](`t:state/0`) just as for a [_state callback_](#state-callback),\nexcept that the action `postpone` is forced to `false` since there is\nno event to postpone.\n\nFor an unsuccessful initialization, `{:stop, reason}`, `{:error, reason}`,\nor `:ignore` should be used; see [`start_link/3,4`](`start_link/3`).","ref":"GenStatem.html#t:init_result/2"},{"type":"type","title":"GenStatem.postpone/0","doc":"Postpone an event to handle it later.\n\nIf `true`, postpones the current event.\nAfter a _state change_ (`next_state !== state`), it is retried.","ref":"GenStatem.html#t:postpone/0"},{"type":"function","title":"GenStatem.receive_response/2","doc":"Receive a request response.\n\nReceive a response corresponding to the request identifier `req_id`.\nThe request must have been made by `send_request/2`\nto the `gen_statem` process.  This function must be called\nfrom the same process from which `send_request/2` was made.\n\n`timeout` specifies how long to wait for a response. If no response is\nreceived within the specified time, this function returns `timeout`.\nAssuming that the server executes on a node supporting aliases (introduced in OTP 24)\nthe request will also be abandoned.  That is, no response will be received\nafter a time-out. Otherwise, a stray response might be received at a later time.\n\nSee [`call/3`](#call-reply) about how the request is handled and the `reply` is sent\nby the `gen_statem` server.\n\nIf the `gen_statem` server process is dead or dies while this function waits for the\nreply, it returns an `error` return with the exit `reason`.\n\nThe difference between `wait_response/2` and `receive_response/2` is that\n`receive_response/2` abandons the request at time-out so that a potential future\nresponse is ignored, while `wait_response/2` does not.","ref":"GenStatem.html#receive_response/2"},{"type":"function","title":"GenStatem.receive_response/3","doc":"","ref":"GenStatem.html#receive_response/3"},{"type":"function","title":"GenStatem.reply/1","doc":"Send one or multiple `call` replies.\n\nThis funcion can be used by a `GenStatem` callback to explicitly send\none or multiple replies to processes waiting for `call` requests' replies,\nwhen it is impractical or impossible to return `t:reply_action/0`s\nfrom a [_state callback_](#state-callback).\n\n> #### Note {: .info }\n>\n> A reply sent with this function is not visible in `m:sys` debug output.","ref":"GenStatem.html#reply/1"},{"type":"function","title":"GenStatem.reply/2","doc":"Send a `call` `reply` to `from`.\n\nThis funcion can be used by a `GenStatem` callback to explicitly send\na reply to a process waiting for a `call` requests' reply,\nwhen it is impractical or impossible to return a `t:reply_action/0`\nfrom a [_state callback_](#state-callback).\n\n> #### Note {: .info }\n>\n> A reply sent with this function is not visible in `m:sys` debug output.","ref":"GenStatem.html#reply/2"},{"type":"type","title":"GenStatem.reply_action/0","doc":"Reply to a [`call/2,3`](`call/3`).\n\nThis _transition action_ can be invoked by returning it from the\n[_state callback_](#state-callback), from [`Module.init/1`](`c:init/1`).\n\nIt does not set any [`transition_option()`](`t:transition_option/0`)\nbut instead replies to a caller waiting for a reply in `call/3`.\n`from` must be the term from argument [`{:call, from}`](`t:event_type/0`)\nin a call to a [_state callback_](#state-callback).\n\nNote that using this action from [`Module.init/1`](`c:init/1`) would be weird\non the border of witchcraft since there has been no earlier call to a\n[_state callback_](#state-callback) in this server.","ref":"GenStatem.html#t:reply_action/0"},{"type":"opaque","title":"GenStatem.reply_tag/0","doc":"A handle that associates a reply to the corresponding request.","ref":"GenStatem.html#t:reply_tag/0"},{"type":"function","title":"GenStatem.reqids_add/3","doc":"Store a request identifier in a colletion.\n\nStores `req_id` and associates a `label` with the request identifier\nby adding this information to `req_id_collection` and returning\nthe resulting request identifier collection.","ref":"GenStatem.html#reqids_add/3"},{"type":"function","title":"GenStatem.reqids_new/0","doc":"","ref":"GenStatem.html#reqids_new/0"},{"type":"function","title":"GenStatem.reqids_size/1","doc":"Return the number of request identifiers in `req_id_collection`.","ref":"GenStatem.html#reqids_size/1"},{"type":"function","title":"GenStatem.reqids_to_list/1","doc":"Send one or multiple `call` replies.\n\nThis funcion can be used by a `GenStatem` callback to explicitly send\none or multiple replies to processes waiting for `call` requests' replies,\nwhen it is impractical or impossible to return `t:reply_action/0`s\nfrom a [_state callback_](#state-callback).\n\n> #### Note {: .info }\n>\n> A reply sent with this function is not visible in `m:sys` debug output.","ref":"GenStatem.html#reqids_to_list/1"},{"type":"opaque","title":"GenStatem.request_id/0","doc":"An opaque request identifier. See `send_request/2` for details.","ref":"GenStatem.html#t:request_id/0"},{"type":"opaque","title":"GenStatem.request_id_collection/0","doc":"An opaque collection of request identifiers (`t:request_id/0`).\n\nEach request identifier can be associated with\na label chosen by the user.  For more information see `reqids_new/0`.","ref":"GenStatem.html#t:request_id_collection/0"},{"type":"type","title":"GenStatem.response_timeout/0","doc":"Response time-out for an asynchronous call.\n\nUsed to set a time limit on how long to wait for a response using either\n`receive_response/2`, `receive_response/3`, `wait_response/2`, or\n`wait_response/3`.  The time unit used is `millisecond`.\n\n Currently valid values:\n\n- **`0..4294967295`** - Time-out relative to current time in milliseconds.\n\n- **`infinity`** - Infinite time-out. That is,\n  the operation will never time out.\n\n- **`{abs, Timeout}`** - An absolute\n  [Erlang monotonic time](`:erlang.monotonic_time/1`)\n  time-out in milliseconds.  That is, the operation will time out when\n  [`:erlang.monotonic_time(millisecond)`](`:erlang.monotonic_time/1`)\n  returns a value larger than or equal to `Timeout`.\n  `Timeout` is not allowed to identify a time further into the future\n  than `4294967295` milliseconds.  Specifying the time-out\n  using an absolute value is especially handy when you have\n  a deadline for responses corresponding to a complete collection\n  of requests (`t:request_id_collection/0`), since you do not have to\n  recalculate the relative time until the deadline over and over again.","ref":"GenStatem.html#t:response_timeout/0"},{"type":"function","title":"GenStatem.send_request/2","doc":"Send an asynchronous `call` request.\n\nSends `request` to the `GenStatem` process identified by `server_ref`\nand returns a request identifier `req_id`.\n\nThe return value `req_id` shall later be used with `receive_response/2`,\n`wait_response/2`, or `check_response/2` to fetch the actual result\nof the request.  Besides passing the request identifier directly\nto these functions, it can also be stored in a request identifier\ncollection using `reqids_add/3`. Such a collection of request identifiers\ncan later be used in order to get one response corresponding to a\nrequest in the collection by passing the collection as argument to\n`receive_response/3`, `wait_response/3`, or `check_response/3`.  If you\nare about to store the request identifier in a collection, you may want\nto consider using `send_request/4` instead.\n\nThe call\n`GenStatem.send_request(server_ref, request) |> GenStatem.wait_response(timeout)`\n can be seen as equivalent to [`GenStatem.call(server, request, timeout)`](`call/3`),\nignoring the error handling.\n\nSee [`call/3`](#call-reply) about how the request is handled\nand the `reply` is sent by the `GenStatem` server.\n\nThe server's `reply` is returned by one of the [`receive_response/1,2`](`receive_response/2`),\n[`wait_response/1,2`](`wait_response/2`), or `check_response/2` functions.","ref":"GenStatem.html#send_request/2"},{"type":"function","title":"GenStatem.send_request/4","doc":"Send an asynchronous `call` request and add it to a request identifier collection.\n\nSends `request` to the `GenStatem` process identified by `server_ref`.\nThe `label` will be associated with the request identifier\nof the operation and added to the returned request identifier collection\n`new_req_id_collection`.  The collection can later be used in order to\nget one response corresponding to a request in the collection\nby passing the collection as argument to `receive_response/3`,\n`wait_response/3`, or `check_response/3`.\n\nThe same as calling\n[`reqids_add(​`](`reqids_add/3`)[`send_request(server_ref, request),\n`](`send_request/2`)[`label, req_id_collection)`](`reqids_add/3`),\nbut slightly more efficient.","ref":"GenStatem.html#send_request/4"},{"type":"type","title":"GenStatem.server_name/0","doc":"Server name specification: `:local`, `:global`, or `:via` registered.\n\nName specification to use when starting a `GenStatem` server.\nSee `start_link/3` and `t:server_ref/0` below.","ref":"GenStatem.html#t:server_name/0"},{"type":"type","title":"GenStatem.server_ref/0","doc":"Server specification: `t:pid/0` or registered `t:server_name/0`.\n\nTo be used in [`call/2,3`](`call/3`) to specify the server.\n\nIt can be:\n\n- **`pid() | local_name`** - The `GenStatem` is locally registered.\n\n- **`{name, node}`** - The `GenStatem` is locally registered\non another node.\n\n- **`{:global, global_name}`** - The `GenStatem` is globally registered\nin `m:global`.\n\n- **`{:via, reg_mod, via_name}`** - The `GenStatem` is registered\nin an alternative process registry.  The registry callback module\n`reg_mod` is to export functions `register_name/2`, `unregister_name/1`,\n`whereis_name/1`, and `send/2`, which are to behave like\nthe corresponding functions in `m:global`.\nThus, `{:via, :global, global_name}` is the same as `{:global, global_name}`.","ref":"GenStatem.html#t:server_ref/0"},{"type":"function","title":"GenStatem.start/3","doc":"Starts a `GenStatem` process without links (outside of a supervision tree).\n\nSee `start_link/3` for more information.","ref":"GenStatem.html#start/3"},{"type":"function","title":"GenStatem.start_link/3","doc":"Start a `GenStatem` process linked to the current process.\n\nThis is often used to start the `GenStatem` as part of a supervision tree.\n\nOnce the server is started, the `c:init/1` function of the given `module` is\ncalled with `init_arg` as its argument to initialize the server. To ensure a\nsynchronized start-up procedure, this function does not return until `c:init/1`\nhas returned.\n\nNote that a `GenStatem` started with `start_link/3` is linked to the\nparent process and will exit in case of crashes from the parent. The GenStatem\nwill also exit due to the `:normal` reasons in case it is configured to trap\nexits in the `c:init/1` callback.","ref":"GenStatem.html#start_link/3"},{"type":"function","title":"Options - GenStatem.start_link/3","doc":"* `:name` - used for name registration as described in the \"Name\n    registration\" section in the documentation for `GenStatem`\n\n  * `:timeout` - if present, the server is allowed to spend the given number of\n    milliseconds initializing or it will be terminated and the start function\n    will return `{:error, :timeout}`\n\n  * `:debug` - if present, the corresponding function in the [`:sys` module](`:sys`) is invoked\n\n  * `:spawn_opt` - if present, its value is passed as options to the\n    underlying process as in `Process.spawn/4`\n\n  * `:hibernate_after` - if present, the GenStatem process awaits any message for\n    the given number of milliseconds and if no message is received, the process goes\n    into hibernation automatically (by calling `:proc_lib.hibernate/3`).","ref":"GenStatem.html#start_link/3-options"},{"type":"function","title":"Return values - GenStatem.start_link/3","doc":"If the server is successfully created and initialized, this function returns\n`{:ok, pid}`, where `pid` is the PID of the server. If a process with the\nspecified server name already exists, this function returns\n`{:error, {:already_started, pid}}` with the PID of that process.\n\nIf the `c:init/1` callback fails with `reason`, this function returns\n`{:error, reason}`. Otherwise, if it returns `{:stop, reason}`\nor `:ignore`, the process is terminated and this function returns\n`{:error, reason}` or `:ignore`, respectively.","ref":"GenStatem.html#start_link/3-return-values"},{"type":"type","title":"GenStatem.start_mon_ret/0","doc":"Return value from the [`start_monitor/3,4`](`start_monitor/3`) functions.\n\nAs for [`start_link/4`](#start-return-values) but a successful return\nwraps the process ID and the [monitor reference](`:erlang.monitor/2`) in a\n`{:ok, {`[`pid()`](`t:pid/0`)`, `[`reference()`](`t:reference/0`)`}}`\ntuple.","ref":"GenStatem.html#t:start_mon_ret/0"},{"type":"function","title":"GenStatem.start_monitor/3","doc":"Starts a `GenStatem` process monitored and registered, but not linked.\n\nSee `start_link/3` for more information.","ref":"GenStatem.html#start_monitor/3"},{"type":"type","title":"GenStatem.start_opt/0","doc":"Server [start options](#start-options) for the\n[`start/3,4`](`start/3`), [`start_link/3,4`](`start_link/3`),\nand [`start_monitor/3,4`](`start_monitor/3`) functions.\n\nSee [`start_link/4`](#start-options).","ref":"GenStatem.html#t:start_opt/0"},{"type":"type","title":"GenStatem.start_ret/0","doc":"[Return value](#start-return-values) from the [`start/3,4`](`start/3`)\nand [`start_link/3,4`](`start_link/3`) functions.\n\nSee [`start_link/4`](#start-return-values).","ref":"GenStatem.html#t:start_ret/0"},{"type":"type","title":"GenStatem.state/0","doc":"State name or state term.\n\nIf the [_callback mode_](`t:callback_mode/0`) is `handle_event_function`,\nthe state can be any term. After a _state change_ (`next_state !== state`),\nall postponed events are retried.\n\nComparing two states for strict equality is assumed to be a fast operation,\nsince for every _state transition_ the `GenStatem` engine has to deduce\nif it is a  _state change_.\n\n> #### Note {: .info }\n> The smaller the state term, in general, the faster the comparison.\n>\n> Note that if the \"same\" state term is returned for a state transition\n> (or a return action without a `next_state` field is used),\n> the comparison for equality is always fast because that can be seen\n> from the term handle.\n>\n> But if a newly constructed state term is returned,\n> both the old and the new state terms will have to be traversed\n> until an inequality is found, or until both terms\n> have been fully traversed.\n>\n> So it is possible to use large state terms that are fast to compare,\n> but very easy to accidentally mess up. Using small state terms is\n> the safe choice.","ref":"GenStatem.html#t:state/0"},{"type":"type","title":"GenStatem.state_callback_result/2","doc":"Return value from any [_state callback_](#state-callback).\n\n`ActionType` is `t:enter_action/0` if the state callback\nwas called with a [_state enter call_](`t:state_enter/0`),\nand `t:action/0` if the state callback was called with an event.\n\n- **`{keep_state, NewData [, Actions]}`** - The same as\n  `{next_state, CurrentState, NewData [, Actions]}`.\n\n- **`keep_state_and_data | {keep_state_and_data, Actions}`** -\n  The same as `{keep_state, CurrentData [, Actions]}`.\n\n- **`{repeat_state, NewData [, Actions]}`** - If the `GenStatem`\n  runs with [_state enter calls_](`t:state_enter/0`),\n  the _state enter call_ is repeated, see type `t:transition_option/0`.\n  Other than that `{repeat_state, NewData [, Actions]}` is the same as\n  `{keep_state, NewData [, Actions]}`.\n\n- **`repeat_state_and_data | {repeat_state_and_data, Actions}`** -\n  The same as `{repeat_state, CurrentData [, Actions]}`.\n\n- **`{stop, Reason [, NewData]}`** - Terminates the `GenStatem`\n  by calling [`Module.terminate/3`](`c:terminate/3`)\n  with `Reason` and `NewData`, if specified. An exit signal\n  with this reason is sent to linked processes and ports.\n\n- **`stop`** - The same as `{stop, normal}`.\n\n- **`{stop_and_reply, Reason, Replies [, NewData]}`** -\n  Sends all `Replies`, then terminates the `GenStatem`\n  like with `{stop, Reason [, NewData]}`.\n\nAll these terms are tuples or atoms and will be so\nin all future versions of `GenStatem`.","ref":"GenStatem.html#t:state_callback_result/2"},{"type":"type","title":"GenStatem.state_enter/0","doc":"[_Callback mode_](`t:callback_mode/0`) modifier\nfor _state enter calls_: the atom `:state_enter`.\n\nBoth _callback modes_ can use _state enter calls_,\nand this is selected by adding this `:state_enter` flag\nto the [_callback mode_](`t:callback_mode/0`) return value from\n[`Module.callback_mode/0`](`c:callback_mode/0`).\n\nIf [`Module.callback_mode/0`](`c:callback_mode/0`) returns\na list containing `:state_enter`, the `GenStatem` engine will,\nat every _state change_, that is; `next_state !== current_state`,\ncall the [_state callback_](#state-callback) with arguments\n`(enter, OldState, Data)` or `(enter, OldState, State, Data)`,\ndepending on the [_callback mode_](`t:callback_mode/0`).\n\nThis may look like an event but is really a call performed\nafter the previous [_state callback_](#state-callback) returned,\nand before any event is delivered to the new\n[_state callback_](#state-callback).\nSee [`Module.state_name/3`](`c:state_name/3`) and\n[`Module.handle_event/4`](`c:handle_event/4`).  A _state enter call_\nmay be repeated without doing a _state change_ by returning\na [`repeat_state`](`t:state_callback_result/2`) or\n[`repeat_state_and_data`](`t:state_callback_result/2`) action\nfrom the _state callback_.\n\nIf [`Module.callback_mode/0`](`c:callback_mode/0`) does not return\na list containing `:state_enter`, no _state enter calls_ are done.\n\nIf [`Module.code_change/4`](`c:code_change/4`) should transform the state,\nit is regarded as a state rename and not a _state change_,\nwhich will not cause a _state enter call_.\n\nNote that a _state enter call_ **will** be done right before entering\nthe initial state, which may be seen as a state change from no state\nto the initial state. In this case `OldState =:= State`,\nwhich cannot happen for a subsequent state change,\nbut will happen when repeating the _state enter call_.","ref":"GenStatem.html#t:state_enter/0"},{"type":"type","title":"GenStatem.state_enter_result/1","doc":"","ref":"GenStatem.html#t:state_enter_result/1"},{"type":"type","title":"GenStatem.state_enter_result/2","doc":"Return value from a [_state callback_](#state-callback)\nafter a _state enter call_.\n\n`state` is the current state and it cannot be changed since the state callback\n was called with a [_state enter call_](`t:state_enter/0`).\n\n- **`{:next_state, state, new_data [, actions]}`** -\n  The `GenStatem` does a state transition to `state`, which has to be\n  equal to the current state, sets `new_data`, and executes all `actions`.","ref":"GenStatem.html#t:state_enter_result/2"},{"type":"type","title":"GenStatem.state_name/0","doc":"State name in [_callback mode_](`t:callback_mode/0`) `state_functions`.\n\nIf the [_callback mode_](`t:callback_mode/0`) is `state_functions`,\nthe state must be an atom. After a _state change_ (`next_state !== state`),\nall postponed events are retried. Note that the state `terminate`\nis not possible to use since it would collide with the optional\ncallback function [`Module.terminate/3`](`c:terminate/3`).","ref":"GenStatem.html#t:state_name/0"},{"type":"callback","title":"GenStatem.state_name/3","doc":"[_State callback_](#state-callback) in\n[_callback mode_](`t:callback_mode/0`) `:state_functions`.\n\nState callback that handles all events in state `state_name`, where\n[`state_name :: state_name()`](`t:state_name/0`)\nhas to be an `t:atom/0`.\n\n`state_name` cannot be `terminate` since that would collide\nwith the callback function [`Module.terminate/3`](`c:terminate/3`).\n\nBesides that when doing a [_state change_](#state-callback)\nthe next state always has to be an `t:atom/0`,\nthis function is equivalent to\n[`Module.handle_event(event_type, event_content,\n__ENV__.function, data)`](`c:handle_event/4`),\nwhich is the [_state callback_](#state-callback) in\n[_callback mode_](`t:callback_mode/0`) `handle_event_function`.","ref":"GenStatem.html#c:state_name/3"},{"type":"type","title":"GenStatem.state_timeout/0","doc":"How long to wait in the current state.\n\nStarts a timer set by `t:timeout_action/0`, or\n`{:state_timeout, time, event_content [, options]}`.\n\nWhen the timer expires an event of `t:event_type/0` `:state_timeout`\nwill be generated. See `:erlang.start_timer/4` for how `time`\nand [`options`](`t:timeout_option/0`) are interpreted. Future\n`:erlang.start_timer/4` `options` will not necessarily be supported.\n\nA _state change_ cancels this timer, if it is running. That is, if the\n`t:timeout_action/0` that starts this timer is part of a list of\n`t:action/0`s for a _state change_, `next_state !== current_state`,\nthe timer runs in the **`next_state`**.\n\nIf the state machine stays in that new state, now the current state,\nthe timer will run until it expires, which creates the time-out event.\nIf the state machine changes states from the now current state,\nthe timer is cancelled.  During the _state change_ from\nthe now current state, a new _state time-out_ may be started\nfor the next **`next_state`**.\n\nIf the `t:timeout_action/0` that starts this timer\nis part of a list of `t:action/0`s for a _state transition_\nthat is not a _state change_, the timer runs in the current state.\n\nIf `Time` is `infinity`, no timer is started,\nas it never would expire anyway.\n\nIf `Time` is relative and `0` no timer is actually started,\ninstead the the time-out event is enqueued to ensure\nthat it gets processed before any not yet received external event.\n\nSetting this timer while it is running will restart it\nwith the new time-out value.  Therefore it is possible\nto cancel this time-out by setting it to `infinity`.\nIt can also be cancelled more explicitly with\n[`{state_timeout, cancel}`](`t:timeout_cancel_action/0`).\n\nThe timer `EventContent` can be updated with the\n[`{state_timeout, update, NewEventContent}`](`t:timeout_update_action/0`)\naction without affecting the time of expiry.","ref":"GenStatem.html#t:state_timeout/0"},{"type":"function","title":"GenStatem.stop/3","doc":"Synchronously stops the state machine with the given `reason`.\n\nThe `c:terminate/3` callback of the given `server` will be invoked before\nexiting. This function returns `:ok` if the server terminates with the\ngiven reason; if it terminates with another reason, the call exits.\n\nThis function keeps OTP semantics regarding error reporting.\nIf the reason is any other than `:normal`, `:shutdown` or\n`{:shutdown, _}`, an error report is logged.","ref":"GenStatem.html#stop/3"},{"type":"callback","title":"GenStatem.terminate/3","doc":"Handle state machine termination.\n\nThis function is called by a `GenStatem` when it is about to terminate.\nIt is to be the opposite of [`Module.init/1`](`c:init/1`)\nand do any necessary cleaning up.  When it returns, the `GenStatem`\nterminates with `reason`.  The return value is ignored.\n\n`reason` is a term denoting the stop reason and [`state`](`t:state/0`)\nis the internal state of the `GenStatem`.\n\n`reason` depends on why the `GenStatem` is terminating. If it is because\nanother callback function has returned, a stop tuple `{:stop, reason}` in\n[`actions`](`t:action/0`), `reason` has the value specified in that tuple.\nIf it is because of a failure, `reason` is the error reason.\n\nIf the `GenStatem` is part of a supervision tree and is ordered by its\nsupervisor to terminate, this function is called with `reason = :shutdown`\nif both the following conditions apply:\n\n- The `GenStatem` process has been set to trap exit signals.\n- The shutdown strategy as defined in the supervisor's\n  child specification is an integer time-out value, not `:brutal_kill`.\n\nEven if the `GenStatem` is _not_ part of a supervision tree,\nthis function is called if it receives an `:EXIT` message\nfrom its parent. `reason` is the same as in the `:EXIT` message.\n\nIf the `GenStatem` process is not set up to trap exit signals it is\nimmediately terminated, just like any process, and this function is not called.\n\nNotice that for any other reason than `:normal`, `:shutdown`, or\n`{:shutdown, term}`, the `GenStatem` is assumed to terminate\nbecause of an error and an error report is issued using `m:logger`.\n\nWhen the `GenStatem` process exits, an exit signal\nwith the same reason is sent to linked processes and ports,\njust as for any process.","ref":"GenStatem.html#c:terminate/3"},{"type":"type","title":"GenStatem.timeout_action/0","doc":"Event time-out, generic time-outs or state time-out.\n\nThese _transition actions_ can be invoked by returning them from the\n[_state callback_](#state-callback), from\n[`Module.init/1`](`c:init/1`).\n\nThese time-out actions sets time-out [transition options](`t:transition_option/0`).\n\n- **`time`** - Short for `{:timeout, time, time}`, that is, the time-out message\n  is the time-out time. This form exists to allow the\n  [_state callback_](#state-callback) return value\n  `{:next_state, next_state, new_data, time}`\n\n- **`{:timeout, time, event_content, [options]}`** -\n  Sets the `t:transition_option/0` `t:event_timeout/0` to `time`\n  with `event_content`, and time-out options\n  [`options`](`t:timeout_option/0`).\n\n- **`{{:timeout, name}, time, event_content, [options]}`** -\n  Sets the `t:transition_option/0` `t:generic_timeout/0` to `time`\n  for time-out `name` with `event_content`, and time-out options\n  [`options`](`t:timeout_option/0`).\n\n- **`{:state_timeout, time, event_content, [options]}`** -\n  Sets the `t:transition_option/0` `t:state_timeout/0` to `time`\n  with `event_content`, and time-out options [`Options`](`t:timeout_option/0`).","ref":"GenStatem.html#t:timeout_action/0"},{"type":"type","title":"GenStatem.timeout_cancel_action/0","doc":"Clearer way to cancel a time-out than the original\nsetting it to ':infinity'.\n\nIt has always been possible to cancel a time-out using\n`t:timeout_action/0` with `time = :infinity`, since setting a new\ntime-out time overrides a running timer, and since setting the time\nto `:infinity` is optimized to not setting a timer (that never\nwill expire).  Using this action shows the intention more clearly.","ref":"GenStatem.html#t:timeout_cancel_action/0"},{"type":"type","title":"GenStatem.timeout_event_type/0","doc":"[Event time-out](`t:event_timeout/0`),\n[generic time-out](`t:generic_timeout/0`),\nor [state time-out](`t:state_timeout/0`).\n\nThe time-out event types that the state machine can generate\nfor itself with the corresponding `t:timeout_action/0`s:\n\n| Time-out type     | Action                         | Event type        |\n|-------------------|--------------------------------|-------------------|\n| Event time-out    | `{:timeout, time, ...}`         | `:timeout`         |\n| Generic time-out  | `{{:timeout, name}, time, ...}` | `{:timeout, name}` |\n| State time-out    | `{:state_timeout, time, ...}`   | `:state_timeout`   |\n\nIn short; the action to set a time-out with\n[`event_type`](`t:timeout_event_type/0`) is `{event_type, time, ...}`.","ref":"GenStatem.html#t:timeout_event_type/0"},{"type":"type","title":"GenStatem.timeout_option/0","doc":"Time-out timer start option, to select absolute time of expiry.\n\nIf `Abs` is `true` an absolute timer is started,\nand if it is `false` a relative, which is the default.\nSee [`:erlang.start_timer/4`](`:erlang.start_timer/4`) for details.","ref":"GenStatem.html#t:timeout_option/0"},{"type":"type","title":"GenStatem.timeout_update_action/0","doc":"Update the `event_content` without affecting the time of expiry.\n\nSets a new `event_content` for a running time-out timer. See\n[timeout_action()](`t:timeout_action/0`) for how to start a time-out.\n\nIf no time-out of this type is active, instead inserts the time-out event\njust like when starting a time-out with relative `time = 0`.  This is a\ntime-out autostart with immediate expiry, so there will be noise for example\nif a generic time-out name was misspelled.","ref":"GenStatem.html#t:timeout_update_action/0"},{"type":"type","title":"GenStatem.transition_option/0","doc":"_State transition_ options set by [actions](`t:action/0`).\n\nThese determine what happens during the _state transition_.\nThe _state transition_ takes place when the\n[_state callback_](#state-callback) has processed an event\nand returns. Here are the sequence of steps for a _state transition_:\n\n1. All returned [actions](`t:action/0`) are processed\n   in order of appearance.  In this step all replies generated\n   by any `t:reply_action/0` are sent.  Other actions set\n   `t:transition_option/0`s that come into play in subsequent steps.\n\n2. If [_state enter calls_](`t:state_enter/0`) are used,\n   it is either the initial state or one of the callback results\n   [`repeat_state`](`t:state_callback_result/2`) or\n   [`repeat_state_and_data`](`t:state_callback_result/2`) is used the\n   `GenStatem` engine calls the current _state callback_ with arguments\n   [`(enter, State, Data)`](`t:state_enter/0`) or\n   [`(enter, State, State, Data)`](`t:state_enter/0`) (depending on\n   [_callback mode_](`t:callback_mode/0`)) and when it returns\n   starts again from the top of this sequence.\n\n   If [_state enter calls_](`t:state_enter/0`) are used,\n   and the state changes, the `GenStatem` engine calls\n   the new _state callback_ with arguments\n   [`(enter, OldState, Data)`](`t:state_enter/0`) or\n   [`(enter, OldState, State, Data)`](`t:state_enter/0`) (depending on\n   [_callback mode_](`t:callback_mode/0`)) and when it returns\n   starts again from the top of this sequence.\n\n3. If `t:postpone/0` is `true`, the current event is postponed.\n\n4. If this is a _state change_, the queue of incoming events is reset\n   to start with the oldest postponed.\n\n5. All events stored with `t:action/0` `next_event` are inserted\n   to be processed before previously queued events.\n\n6. Time-out timers `t:event_timeout/0`, `t:generic_timeout/0` and\n   `t:state_timeout/0` are handled.  Time-outs with zero time\n   are guaranteed to be delivered to the state machine\n   before any external not yet received event so if there is\n   such a time-out requested, the corresponding time-out zero event\n   is enqueued as the newest received event; that is after\n   already queued events such as inserted and postponed events.\n\n   Any event cancels an `t:event_timeout/0` so a zero time event time-out\n   is only generated if the event queue is empty.\n\n   A _state change_ cancels a `t:state_timeout/0` and any new transition\n   option of this type belongs to the new state, that is;\n   a `t:state_timeout/0` applies to the state the state machine enters.\n\n7. If there are enqueued events the\n   [_state callback_](#state-callback) for the possibly\n   new state is called with the oldest enqueued event, and we start again\n   from the top of this sequence.\n\n8. Otherwise the `GenStatem` goes into `receive` or hibernation\n   (if `t:hibernate/0` is `true`) to wait for the next message.\n   In hibernation the next non-system event awakens the `GenStatem`,\n   or rather the next incoming message awakens the `GenStatem`,\n   but if it is a system event it goes right back into hibernation.\n   When a new message arrives the\n   [_state callback_](#state-callback) is called with\n   the corresponding event, and we start again\n   from the top of this sequence.\n\n> #### Note {: .info }\n> The behaviour of a time-out zero (a time-out with time `0`)\n> differs subtly from Erlang's `receive ... after 0 ... end`.\n>\n> The latter receives one message if there is one,\n> while using the `t:timeout_action/0` `{timeout, 0}` does not\n> receive any external event.\n>\n> `m:gen_server`'s time-out works like Erlang's\n> `receive ... after 0 ... end`, in contrast to `GenStatem`.","ref":"GenStatem.html#t:transition_option/0"},{"type":"function","title":"GenStatem.wait_response/2","doc":"Wait for a request response.\n\nWaits for the response to the request identifier `req_id`.  The request\nmust have been made by `send_request/2` to the `GenStatem` process.\nThis function must be called from the same process from which\n`send_request/2` was called.\n\n`wait_time` specifies how long to wait for a reply. If no reply is received\nwithin the specified time, the function returns `timeout` and no cleanup is done,\nThus the function can be invoked repeatedly until a reply is returned.\n\nSee [`call/3`](#call-reply) about how the request is handled and the `Reply` is\nsent by the `GenStatem` server.\n\nIf the `GenStatem` server process is dead or dies while this function waits for\nthe reply, it returns an `error` return with the exit `reason`.\n\nThe difference between `receive_response/2` and `wait_response/2` is that\n`receive_response/2` abandons the request at time-out so that a potential future\nresponse is ignored, while `wait_response/2` does not.","ref":"GenStatem.html#wait_response/2"},{"type":"function","title":"GenStatem.wait_response/3","doc":"Wait for any request response in a collection.\n\nWaits for a response in `req_id_collection`.  All request identifiers\nof `req_id_collection` must correspond to requests that have been made\nusing `send_request/2` or `send_request/4`, and all requests\nmust have been made by the process calling this function.\n\nThe `label` in the response is the `label` associated with\nthe request identifier that the response corresponds to.\nThe `label` of a request identifier is associated\nwhen [adding the request id](`reqids_add/3`) to a collection,\nor when sending the request using `send_request/4`.\n\nCompared to `wait_response/2`, the returned result or exception\nassociated with a specific request identifier will be wrapped\nin a 3-tuple `{response, label, new_req_id_collection}`.\n`response` is the value that would have been produced\nby `wait_response/2`, `label` is the value associated with\nthe specific [request identifier](`t:request_id/0`)\nand `new_req_id_collection` is a possibly modified\nrequest identifier collection.\n\nIf `req_id_collection` is empty, `no_request` is returned.\n\nIf no response is received before `wait_time` has expired,\n`timeout` is returned.  It is valid to continue waiting\nfor a response as many times as needed up until a response\nhas been received and completed by `check_response()`,\n`receive_response()`, or `wait_response()`.\n\nThe difference between `receive_response/3` and `wait_response/3`\nis that `receive_response/3` abandons requests at time-out\nso that potential future responses are ignored, while\n`wait_response/3` does not.\n\nIf `delete` is `true`, the association with `label` has been deleted\nfrom `req_id_collection` in the resulting `new_req_id_collection`. If\n`delete` is `false`, `new_req_id_collection` will equal `req_id_collection`.\nNote that deleting an association is not for free and that a collection\ncontaining already handled requests can still be used by subsequent calls to\n`wait_response/3`, `check_response/3`, and `receive_response/3`.\n\nHowever, without deleting handled associations, the above calls will not be\nable to detect when there are no more outstanding requests to handle, so you\nwill have to keep track of this some other way than relying on a `:no_request`\nreturn. Note that if you pass a collection only containing associations of\nalready handled or abandoned requests to this function, it will always block\nuntil `wait_time` expires and then return `:timeout`.","ref":"GenStatem.html#wait_response/3"},{"type":"exception","title":"GenStatem.GenError","doc":"","ref":"GenStatem.GenError.html"},{"type":"extras","title":"GenStatem","doc":"# GenStatem\n\n[![Coverage Status](https://coveralls.io/repos/github/gilbertwong96/gen_statem/badge.svg?branch=main)](https://coveralls.io/github/gilbertwong96/gen_statem?branch=main)[![Build Status](https://github.com/phoenixframework/phoenix/workflows/CI/badge.svg)](https://github.com/gilbertwong96/gen_statem/actions/workflows/ci.yml) [![Hex.pm](https://img.shields.io/hexpm/v/phoenix.svg)](https://hex.pm/packages/gen_statem) [![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/gen_statem)\n\nAn Elixir implementation of the `gen_statem` behavior from Erlang/OTP.","ref":"readme.html"},{"type":"extras","title":"Features - GenStatem","doc":"- Co-located state code (`state_functions` mode)\n- Arbitrary term state (`handle_event_function` mode)\n- Event postponing\n- Self-generated events\n- State time-outs\n- Multiple generic named time-outs\n- Absolute time-out time\n- Automatic state enter calls\n- Reply from other state than the request\n- Multiple replies\n- Changing the callback module","ref":"readme.html#features"},{"type":"extras","title":"Installation - GenStatem","doc":"Add `gen_statem` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:gen_statem, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - GenStatem","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Callback Modes - GenStatem","doc":"Two callback modes are supported:\n\n1. `state_functions` - For finite-state machines where states must be atoms and each state has its own callback function\n2. `handle_event_function` - Allows states to be any term and uses a single callback function for all states","ref":"readme.html#callback-modes"},{"type":"extras","title":"Example Pushbutton Implementation - GenStatem","doc":"Here's a simple pushbutton example using `state_functions` mode:\n\n```elixir\ndefmodule PushButton do\n  use GenStatem\n\n  defp name, do: :pushbutton_statem\n\n  # API\n  def start, do: GenStatem.start(__MODULE__, [], name: name())\n  def push, do: GenStatem.call(name(), :push)\n  def get_count, do: GenStatem.call(name(), :get_count)\n  def stop, do: GenStatem.stop(name())\n\n  # Callbacks\n  def init([]) do\n    {:ok, :off, 0}\n  end\n\n  def callback_mode(), do: :state_functions\n\n  # State callbacks\n  def off({:call, from}, :push, data) do\n    {:next_state, :on, data + 1, [{:reply, from, :on}]}\n  end\n  def off(event_type, event_content, data) do\n    handle_event(event_type, event_content, data)\n  end\n\n  def on({:call, from}, :push, data) do\n    {:next_state, :off, data, [{:reply, from, :off}]}\n  end\n  def on(event_type, event_content, data) do\n    handle_event(event_type, event_content, data)\n  end\n\n  # Common event handler\n  defp handle_event({:call, from}, :get_count, data) do\n    {:keep_state, data, [{:reply, from, data}]}\n  end\n  defp handle_event(_event_type, _event_content, data) do\n    {:keep_state, data}\n  end\nend\n```","ref":"readme.html#example-pushbutton-implementation"},{"type":"extras","title":"Starting the Server - GenStatem","doc":"Start the server using one of:\n\n```elixir\nGenStatem.start(module, init_arg, options)\nGenStatem.start_link(module, init_arg, options)\nGenStatem.start_monitor(module, init_arg, options)\n```","ref":"readme.html#starting-the-server"},{"type":"extras","title":"Making Calls - GenStatem","doc":"Synchronous call:\n```elixir\nGenStatem.call(server_ref, request, timeout \\\\ :infinity)\n```\n\nAsynchronous cast:\n```elixir\nGenStatem.cast(server_ref, msg)\n```","ref":"readme.html#making-calls"},{"type":"extras","title":"Transition Actions - GenStatem","doc":"State callbacks can return actions like:\n\n- `{:next_state, new_state, new_data}`\n- `{:reply, from, reply}`\n- `{:postpone, true}`\n- `{:timeout, time, content}`\n- `{:state_timeout, time, content}`\n- And more...","ref":"readme.html#transition-actions"},{"type":"extras","title":"Comparison with GenServer - GenStatem","doc":"| Feature          | GenStatem | GenServer |\n|-----------------|-----------|-----------|\n| State handling  | Explicit states with callbacks | Implicit in server data |\n| Event handling  | Built-in event queue and postponing | Manual handling |\n| Performance     | Optimized for state transitions | General purpose |\n| Use cases       | Complex workflows, protocols | General server needs |","ref":"readme.html#comparison-with-genserver"},{"type":"extras","title":"Troubleshooting - GenStatem","doc":"","ref":"readme.html#troubleshooting"},{"type":"extras","title":"Common Issues - GenStatem","doc":"1. **State not changing**: Ensure you return `{:next_state, ...}` not `{:keep_state, ...}`\n2. **Events not processed**: Check if events are being postponed accidentally\n3. **Timeouts not firing**: Verify no other events are canceling them","ref":"readme.html#common-issues"},{"type":"extras","title":"Documentation - GenStatem","doc":"For complete documentation, see the [module docs](https://hexdocs.pm/gen_statem).","ref":"readme.html#documentation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}